-- This MySQL script calculates design values for a PM hot-spot
-- analysis for the 24-hour PM2.5 NAAQS using dispersion model
-- results and air quality monitoring data provided by the user.
-- This calculation is described as a 24-hour PM2.5 "second tier"
-- analysis in the EPA guidance document "Transportation
-- Conformity Guidance for Quantitative Hot-spot Analyses in
-- PM2.5 and PM10 Nonattainment and Maintenance Areas."  See the
-- guidance for additional information about how and when to
-- complete this calculation.
-- Script results are reported in the output table:
-- `receptorranking' with the average concentration given for
-- each receptor.  Results are ranked in descending order with
-- receptor with the highest concentration listed first.
-- This script may take several minutes to run.
--
-- ***********************************************************
-- For script to operate:
-- The user *must* create output database `design_values`.
-- The user *must* have modeled data in a .csv file called
-- `modeled_data.csv` located in folder c:\design values.
-- The user *must* have monitored data in a .csv file called
-- `monitor_data.csv` located in folder c:\design values.
-- ***********************************************************
--
-- Script version: September 2013.  Corrects bug that occurred
-- when modeled values were 10.0 or greater.

Use design_values;

drop table IF EXISTS
	activitytype,
	bundletracking,
	modeled,
	monitored,
	movesactivityoutput,
	moveserror,
	moveseventlog,
	movesoutput,
	movesrun,
	movestablesused,
	movesworkersused,
	rateperdistance,
	rateperprofile,
	ratepervehicle;

create table `modeled` (
 `x` char(30) default NULL,
 `y` char(30) default NULL,
 `average conc` decimal(10,6),
 `date` char(10)
) ENGINE=MyISAM;

load data infile 'C:/Design Values/Modeled_Data.csv'
into table modeled
fields terminated by ','
optionally enclosed by '"'
escaped by '\\'
lines terminated by '\r\n'
ignore 1 lines;

create table `monitored` (
 `year` char(4) default NULL,
 `quarter` int(1) default NULL,
 `rank` int(1),
 `value` float NOT NULL,
 `countYear` char(4),
 `count` char(3)
) ENGINE=MyISAM;

load data infile 'C:/Design Values/Monitor_Data.csv'
into table monitored
fields terminated by ','
optionally enclosed by '"'
escaped by '\\'
lines terminated by '\r\n'
ignore 1 lines;

Use design_values;

Create table modeledA
Select concat(Substring(x,1,30),
              "x",
              Substring(y,1,30)) as receptorId,
       `average conc`           as avgConc,
       substring(date,1,2)      as yearId,
       substring(date,3,2)      as monthId
From   modeled;

Alter table modeledA add column quarterId int;

Update modeledA set quarterId = 1 where monthId in ( 1, 2, 3);
Update modeledA set quarterId = 2 where monthId in ( 4, 5, 6);
Update modeledA set quarterId = 3 where monthId in ( 7, 8, 9);
Update modeledA set quarterId = 4 where monthId in (10,11,12);

Create   table modeledB
Select   receptorId,
         yearId,
         quarterId,
         count(*)      as n,
         Max(avgConc)  as maxYearQtrAvg
From     modeledA
Group by receptorId,
         yearId,
         QuarterId
Order by receptorId,
         yearId,
         QuarterId;

Create   table modeledC
Select   receptorId,
         quarterId,
         count(*)           as n,
         avg(maxYearQtrAvg) as qtrAvg
From     modeledB
Group by receptorId,
         quarterId
Order by receptorId,
         quarterId;

Create   table F
select   year as yearId,
         rank as rankId
From     monitored
Group by year,
         rank
Order by year,
         rank;

-- Alter table F add column Quarter1 real;
-- Alter table F add column Quarter2 real;
-- Alter table F add column Quarter3 real;
-- Alter table F add column Quarter4 real;

-- Update      F as a Set a.Quarter1 = (Select B.value
--                                      from   monitored as b
--                                     where  B.quarter = 1
--                                       and  B.year    = a.yearId
--                                       and  B.rank    = a.rankId);

-- Update      F as a Set a.Quarter2 = (Select b.value
--                                     from   monitored as b
--                                     where  b.quarter = 2
--                                       and  b.year    = a.yearId
--                                       and  b.rank    = a.rankId);
--
-- Update      F as a Set a.Quarter3 = (Select b.value
--                                     from   monitored as b
--                                     where  b.quarter = 3
--                                       and  b.year    = a.yearId
--                                       and  b.rank    = a.rankId);

-- Update      F as a Set a.Quarter4 = (Select b.value
--                                     from   monitored as b
--                                     where  b.quarter = 4
--                                       and  b.year    = a.yearId
--                                       and  b.rank    = a.rankId);

Create table G
Select distinct receptorId
from   modeledC;

Create table H
Select *
From   F,G;

Create table I
Select *
From   G,
       monitored;

Create table ICopy
Select *
From   I;

create unique index index1 on I        (receptorId, year, quarter, rank);
create unique index index1 on ICopy    (receptorId, year, quarter, rank);
create unique index index1 on modeledC (receptorId, quarterId);

update I as a set a.value = (select b.value
                             from ICopy as b
                             where a.receptorId = b.receptorId
                               and a.year       = b.year
                               and a.quarter    = b.quarter
                               and a.rank       = b.rank)
                          +
                            (Select c.qtrAvg
                             From modeledC as c
                             Where a.receptorId = c.receptorId
                               and a.quarter    = c.quarterId);


Create table J
Select receptorId,
       year    as yearId,
       quarter as quarterId,
       rank    as rankId,
       value
From I;


Alter table  J add column Rank32 int;
Update       J set        Rank32 = 99;

Create table JCopy
Select * from J;

create unique index index1 on J        (receptorId, yearId, quarterId, rankId);
create unique index index1 on JCopy    (receptorId, yearId, quarterId, rankId);


-- Rank32 = 1
Update J     as a set a.Rank32 = 1 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 1 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 1);
-- Rank32 = 2
Update J     as a set a.Rank32 = 2 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 2 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 2);
-- Rank32 = 3
Update J     as a set a.Rank32 = 3 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 3 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 3);
-- Rank32 = 4
Update J     as a set a.Rank32 = 4 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 4 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 4);
-- Rank32 = 5
Update J     as a set a.Rank32 = 5 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 5 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 5);
-- Rank32 = 6
Update J     as a set a.Rank32 = 6 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 6 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 6);
-- Rank32 = 7
Update J     as a set a.Rank32 = 7 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 7 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 7);
-- Rank32 = 8
Update J     as a set a.Rank32 = 8 where a.value =  (Select Max(b.value)
                                                     From   JCopy as b
                                                     Where  b.receptorId = a.receptorId
                                                       and  b.yearId     = a.yearId
                                                       and  b.Rank32     = 99);

Update JCopy as a set a.Rank32 = 8 where a.value = (Select b.value
                                                     From J as b
                                                     Where a.receptorId = b.receptorId
                                                       and a.yearId     = b.yearid
                                                       and a.quarterId  = b.quarterId
                                                       and a.rankId     = b.rankId
                                                       and b.rank32 = 8);


Create table K
Select *
From   J
Where  Rank32 < 99;

Create   table L
Select   countYear,
         cast(count AS Binary) as Icount
From     monitored
Where    length(countYear) > 0;

Alter table L add column Rank498PT int;

Update L set Rank498Pt = 1 where Icount >=  1 and Icount<= 50;
Update L set Rank498Pt = 2 where Icount >= 51 and Icount<=100;
Update L set Rank498Pt = 3 where Icount >=101 and Icount<=150;
Update L set Rank498Pt = 4 where Icount >=151 and Icount<=200;
Update L set Rank498Pt = 5 where Icount >=201 and Icount<=250;
Update L set Rank498Pt = 6 where Icount >=251 and Icount<=300;
Update L set Rank498Pt = 7 where Icount >=301 and Icount<=350;
Update L set Rank498Pt = 8 where Icount >=351 and Icount<=366;

Alter table K add column Rank498Pt int;

Update K as a set a.Rank498Pt = (Select b.Rank498Pt
                                 From   L as b
                                 Where  b.countYear = a.yearId);

Create   table M
Select   yearId,
         receptorId,
         rank32,
         value,
         rank498Pt
From     K
Where    rank32=Rank498Pt
Order by yearId,
         receptorId,
         rank32;

Drop     table if exists N;
Create   table N
Select   receptorId,
    --   count(*) as n,
         avg(value) as ReceptorAvg
From     M
Group by receptorID
Order by receptorID;

Drop     table if exists ReceptorRanking;
Create   table ReceptorRanking
Select   receptorID,
         ReceptorAvg
From     N
Group by ReceptorAvg
Order by ReceptorAvg DESC;

SELECT * FROM `design_values`.`receptorRanking`;


Drop table f, g, h, i, j, k, l, m, n;
Drop table icopy, jcopy;
Drop table modeleda, modeledb, modeledc;
